# -*- coding: utf-8 -*-
"""
Django management command for creating DataSharingConsent records.
"""
import csv
import logging

from django.core.management import BaseCommand

from consent.models import DataSharingConsent
from enterprise.models import EnterpriseCourseEnrollment, EnterpriseCustomer

LOGGER = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Django management command for creating DataSharingConsent records
    """

    def add_arguments(self, parser):
        parser.add_argument(
            '--data-csv',
            action='store',
            dest='data_csv',
            default=None,
            help='Path of csv to read enterprise learner data',
            type=str,
        )

    def handle(self, *args, **options):
        csv_path = options['data_csv']

        LOGGER.info('Started creation of DSC records')

        with open(csv_path) as csv_file:
            reader = csv.DictReader(csv_file)
            for row in reader:
                course_id = row['COURSE_ID']
                enterprise_uuid = row['ENTERPRISE_UUID']
                username = row['USERNAME']

                enterprise_course_enrollment = EnterpriseCourseEnrollment.objects.filter(
                    course_id=course_id,
                    enterprise_customer_user__user_id=row['USER_ID'],
                    enterprise_customer_user__enterprise_customer__uuid=enterprise_uuid,
                ).exists()
                if enterprise_course_enrollment:
                    enterprise_customer = EnterpriseCustomer.objects.get(uuid=enterprise_uuid)
                    DataSharingConsent.objects.update_or_create(
                        username=username,
                        course_id=course_id,
                        enterprise_customer=enterprise_customer,
                        defaults={
                            'granted': True
                        },
                    )
                    LOGGER.info('DSC record created. User: [%s], Course: [%s]', username, course_id)

        LOGGER.info('Finished creation of DSC records')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.22 on 2019-07-24 11:41


import jsonfield.fields

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models

import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0069_auto_20190613_0607'),
    ]

    operations = [
        migrations.CreateModel(
            name='EnterpriseCatalogQuery',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('title', models.CharField(default='All Content', max_length=255)),
                ('content_filter', jsonfield.fields.JSONField(blank=True, default={}, help_text="Query parameters which will be used to filter the discovery service's search/all endpoint results, specified as a JSON object. An empty JSON object means that all available content items will be included in the catalog.", null=True)),
            ],
            options={
                'ordering': ['created'],
                'verbose_name': 'Enterprise Catalog Query',
                'verbose_name_plural': 'Enterprise Catalog Queries',
            },
        ),
        migrations.AddField(
            model_name='enterprisecustomercatalog',
            name='enterprise_catalog_query',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='enterprise_customer_catalogs', to='enterprise.EnterpriseCatalogQuery'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomercatalog',
            name='enterprise_catalog_query',
            field=models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='enterprise.EnterpriseCatalogQuery'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
The Degreed Integrated Channel package.
"""

__version__ = "0.1.0"

default_app_config = ("integrated_channels.degreed.apps."  # pylint: disable=invalid-name
                      "DegreedConfig")



"""
Transmitter for Blackboard content metadata
"""

from integrated_channels.blackboard.client import BlackboardAPIClient
from integrated_channels.integrated_channel.transmitters.content_metadata import ContentMetadataTransmitter


class BlackboardContentMetadataTransmitter(ContentMetadataTransmitter):
    """
    This transmitter transmits exported content metadata to Canvas.
    """

    def __init__(self, enterprise_configuration, client=BlackboardAPIClient):
        """
        Use the ``BlackboardAPIClient`` for content metadata transmission.
        """
        super(BlackboardContentMetadataTransmitter, self).__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )

    def _prepare_items_for_transmission(self, channel_metadata_items):
        # here is a hack right now to send only one item
        # we have to investigate how to handle multiple
        # metadata items since there is no batch course create endpoint in blackboard
        return channel_metadata_items[0]



# -*- coding: utf-8 -*-
"""
Django signal handlers.
"""

from logging import getLogger

from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.db.models.signals import post_delete, post_save, pre_delete, pre_save
from django.dispatch import receiver

from enterprise.api_client.enterprise_catalog import EnterpriseCatalogApiClient
from enterprise.constants import ENTERPRISE_ADMIN_ROLE, ENTERPRISE_LEARNER_ROLE
from enterprise.decorators import disable_for_loaddata
from enterprise.models import (
    EnterpriseAnalyticsUser,
    EnterpriseCatalogQuery,
    EnterpriseCustomerBrandingConfiguration,
    EnterpriseCustomerCatalog,
    EnterpriseCustomerUser,
    PendingEnterpriseCustomerAdminUser,
    PendingEnterpriseCustomerUser,
    SystemWideEnterpriseRole,
    SystemWideEnterpriseUserRoleAssignment,
)
from enterprise.tasks import create_enterprise_enrollment
from enterprise.utils import (
    NotConnectedToOpenEdX,
    create_tableau_user,
    delete_tableau_user,
    delete_tableau_user_by_id,
    get_default_catalog_content_filter,
    track_enrollment,
    unset_enterprise_learner_language,
)

try:
    from student.models import CourseEnrollment
except ImportError:
    CourseEnrollment = None

logger = getLogger(__name__)  # pylint: disable=invalid-name
_UNSAVED_FILEFIELD = 'unsaved_filefield'


@disable_for_loaddata
def handle_user_post_save(sender, **kwargs):  # pylint: disable=unused-argument
    """
    Handle User model changes - checks if pending enterprise customer user record exists and upgrades it to actual link.

    If there are pending enrollments attached to the PendingEnterpriseCustomerUser, then this signal also takes the
    newly-created users and enrolls them in the relevant courses.
    """
    created = kwargs.get("created", False)
    user_instance = kwargs.get("instance", None)

    if user_instance is None:
        return  # should never happen, but better safe than 500 error

    try:
        pending_ecu = PendingEnterpriseCustomerUser.objects.get(user_email=user_instance.email)
    except PendingEnterpriseCustomerUser.DoesNotExist:
        return  # nothing to do in this case

    if not created:
        # existing user changed his email to match one of pending link records - try linking him to EC
        try:
            existing_record = EnterpriseCustomerUser.objects.get(user_id=user_instance.id)
            message_template = "User {user} have changed email to match pending Enterprise Customer link, " \
                               "but was already linked to Enterprise Customer {enterprise_customer} - " \
                               "deleting pending link record"
            logger.info(message_template.format(
                user=user_instance, enterprise_customer=existing_record.enterprise_customer
            ))
            pending_ecu.delete()
            return
        except EnterpriseCustomerUser.DoesNotExist:
            pass  # everything ok - current user is not linked to other ECs

    enterprise_customer_user = EnterpriseCustomerUser.objects.create(
        enterprise_customer=pending_ecu.enterprise_customer,
        user_id=user_instance.id
    )
    pending_enrollments = list(pending_ecu.pendingenrollment_set.all())
    if pending_enrollments:
        def _complete_user_enrollment():
            """
            Complete an Enterprise User's enrollment.

            EnterpriseCustomers may enroll users in courses before the users themselves
            actually exist in the system; in such a case, the enrollment for each such
            course is finalized when the user registers with the OpenEdX platform.
            """
            for enrollment in pending_enrollments:
                enterprise_customer_user.enroll(
                    enrollment.course_id,
                    enrollment.course_mode,
                    cohort=enrollment.cohort_name,
                    source_slug=getattr(enrollment.source, 'slug', None),
                    discount_percentage=enrollment.discount_percentage,
                    sales_force_id=enrollment.sales_force_id,
                )
                track_enrollment('pending-admin-enrollment', user_instance.id, enrollment.course_id)
            pending_ecu.delete()
        transaction.on_commit(_complete_user_enrollment)
    else:
        pending_ecu.delete()


@receiver(pre_save, sender=EnterpriseCustomerBrandingConfiguration)
def skip_saving_logo_file(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    To avoid saving the logo image at an incorrect path, skip saving it.
    """
    if not instance.id and not hasattr(instance, _UNSAVED_FILEFIELD):
        setattr(instance, _UNSAVED_FILEFIELD, instance.logo)
        instance.logo = None


@receiver(post_save, sender=EnterpriseCustomerBrandingConfiguration)
def save_logo_file(sender, instance, **kwargs):            # pylint: disable=unused-argument
    """
    Now that the object is instantiated and instance.id exists, save the image at correct path and re-save the model.
    """
    if kwargs['created'] and hasattr(instance, _UNSAVED_FILEFIELD):
        instance.logo = getattr(instance, _UNSAVED_FILEFIELD)
        delattr(instance, _UNSAVED_FILEFIELD)
        instance.save()


@receiver(post_save, sender=EnterpriseCustomerCatalog, dispatch_uid='default_content_filter')
def default_content_filter(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Set default value for `EnterpriseCustomerCatalog.content_filter` if not already set.
    """
    if kwargs['created'] and not instance.content_filter:
        instance.content_filter = get_default_catalog_content_filter()
        instance.save()


@receiver(post_save, sender=EnterpriseCustomerUser)
def assign_or_delete_enterprise_admin_role(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Assign or delete enterprise_admin role for EnterpriseCustomerUser when updated.
    Create third party analytics user.

    This only occurs if a PendingEnterpriseCustomerAdminUser record exists.
    """
    if instance.user:
        enterprise_admin_role, __ = SystemWideEnterpriseRole.objects.get_or_create(name=ENTERPRISE_ADMIN_ROLE)
        try:
            pending_enterprise_admin_user = PendingEnterpriseCustomerAdminUser.objects.get(
                user_email=instance.user.email,
                enterprise_customer=instance.enterprise_customer,
            )
        except PendingEnterpriseCustomerAdminUser.DoesNotExist:
            pending_enterprise_admin_user = None

        if kwargs['created'] and pending_enterprise_admin_user:
            # EnterpriseCustomerUser record was created and a pending admin user
            # exists, so assign the enterprise_admin role.
            pending_enterprise_admin_user.activate_admin_permissions(
                user=instance.user,
                enterprise_customer=instance.enterprise_customer,
            )
            # Also create the Enterprise admin user in third party analytics application
            create_tableau_user(instance.user.username, instance)
        elif not kwargs['created'] and not instance.linked:
            # EnterpriseCustomerUser record was updated but is not linked, so delete the enterprise_admin role.
            try:
                SystemWideEnterpriseUserRoleAssignment.objects.get(
                    user=instance.user,
                    role=enterprise_admin_role
                ).delete()
                # Also delete the Enterprise admin user in the third party analytics application
                delete_tableau_user(instance)
            except SystemWideEnterpriseUserRoleAssignment.DoesNotExist:
                # Do nothing if no role assignment is present for the enterprise customer user.
                pass
        else:
            logger.info(
                'Could not assign or delete enterprise_admin role for user %s'
                ' due to a PendingEnterpriseCustomerAdminUser record not existing'
                ' or the user not being linked.',
                instance.user.id,
            )


@receiver(post_delete, sender=EnterpriseCustomerUser)
def delete_enterprise_admin_role_assignment(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Delete the associated enterprise admin role assignment record when deleting an EnterpriseCustomerUser record.
    """
    if instance.user:
        enterprise_admin_role, __ = SystemWideEnterpriseRole.objects.get_or_create(name=ENTERPRISE_ADMIN_ROLE)
        SystemWideEnterpriseUserRoleAssignment.objects.filter(
            user=instance.user,
            role=enterprise_admin_role
        ).delete()


@receiver(post_save, sender=EnterpriseCustomerUser)
def assign_or_delete_enterprise_learner_role(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Assign or delete enterprise_learner role for EnterpriseCustomerUser when updated.

    The enterprise_learner role is assigned when a new EnterpriseCustomerUser record is
    initially created and removed when a EnterpriseCustomerUser record is updated and
    unlinked (i.e., soft delete - see ENT-2538).
    """
    if instance.user:
        enterprise_learner_role, __ = SystemWideEnterpriseRole.objects.get_or_create(name=ENTERPRISE_LEARNER_ROLE)
        if kwargs['created']:
            # EnterpriseCustomerUser record was created, so assign the enterprise_learner role
            SystemWideEnterpriseUserRoleAssignment.objects.get_or_create(
                user=instance.user,
                role=enterprise_learner_role
            )
        elif not kwargs['created'] and not instance.linked:
            # EnterpriseCustomerUser record was updated but is not linked, so delete the enterprise_learner role
            try:
                SystemWideEnterpriseUserRoleAssignment.objects.get(
                    user=instance.user,
                    role=enterprise_learner_role
                ).delete()
            except SystemWideEnterpriseUserRoleAssignment.DoesNotExist:
                # Do nothing if no role assignment is present for the enterprise customer user.
                pass


@receiver(post_delete, sender=EnterpriseCustomerUser)
def delete_enterprise_learner_role_assignment(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Delete the associated enterprise learner role assignment record when deleting an EnterpriseCustomerUser record.
    """
    if instance.user:
        enterprise_learner_role, __ = SystemWideEnterpriseRole.objects.get_or_create(name=ENTERPRISE_LEARNER_ROLE)
        try:
            SystemWideEnterpriseUserRoleAssignment.objects.get(
                user=instance.user,
                role=enterprise_learner_role
            ).delete()
        except SystemWideEnterpriseUserRoleAssignment.DoesNotExist:
            # Do nothing if no role assignment is present for the enterprise customer user.
            pass


@receiver(post_save, sender=EnterpriseCustomerUser)
def update_learner_language_preference(sender, instance, created, **kwargs):     # pylint: disable=unused-argument
    """
    Update the language preference of the learner.
    Set the language preference to the value enterprise customer has used as the `default_language`.
    """
    # Unset the language preference when a new learner is linked with the enterprise customer.
    # The middleware in the enterprise will handle the cases for setting a proper language for the learner.
    if created and instance.enterprise_customer.default_language:
        unset_enterprise_learner_language(instance)


@receiver(pre_delete, sender=EnterpriseAnalyticsUser)
def delete_enterprise_analytics_user(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Delete the associated enterprise analytics user in tableau.
    """
    if instance.analytics_user_id:
        delete_tableau_user_by_id(instance.analytics_user_id)


@receiver(post_save, sender=PendingEnterpriseCustomerAdminUser)
def create_pending_enterprise_admin_user(sender, instance, **kwargs):  # pylint: disable=unused-argument
    """
    Creates a PendingEnterpriseCustomerUser when a PendingEnterpriseCustomerAdminUser is created.
    """
    PendingEnterpriseCustomerUser.objects.get_or_create(
        enterprise_customer=instance.enterprise_customer,
        user_email=instance.user_email,
    )


@receiver(post_delete, sender=PendingEnterpriseCustomerAdminUser)
def delete_pending_enterprise_admin_user(sender, instance, **kwargs):  # pylint: disable=unused-argument
    """
    Deletes a PendingEnterpriseCustomerUser when its associated PendingEnterpriseCustomerAdminUser is removed.
    """
    PendingEnterpriseCustomerUser.objects.filter(
        enterprise_customer=instance.enterprise_customer,
        user_email=instance.user_email,
    ).delete()


@receiver(post_save, sender=EnterpriseCatalogQuery)
def update_enterprise_catalog_query(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Sync data changes from Enterprise Catalog Query to the Enterprise Customer Catalog.
    """
    updated_content_filter = instance.content_filter
    logger.info(
        'Running update_enterprise_catalog_query for Catalog Query {} with updated_content_filter {}'.format(
            instance.pk,
            updated_content_filter
        )
    )
    catalogs = instance.enterprise_customer_catalogs.filter(sync_enterprise_catalog_query=True)

    for catalog in catalogs:
        logger.info(
            'update_enterprise_catalog_query is updating catalog {} with the updated_content_filter.'.format(
                catalog.uuid
            )
        )
        catalog.content_filter = updated_content_filter
        catalog.save()


@receiver(post_save, sender=EnterpriseCustomerCatalog)
def update_enterprise_catalog_data(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Send data changes to Enterprise Catalogs to the Enterprise Catalog Service.
    """
    catalog_uuid = instance.uuid
    try:
        catalog_client = EnterpriseCatalogApiClient()
        if kwargs['created']:
            response = catalog_client.get_enterprise_catalog(
                catalog_uuid=catalog_uuid,
                # Suppress 404 exception on create since we do not expect the catalog
                # to exist yet in enterprise-catalog
                should_raise_exception=False,
            )
        else:
            response = catalog_client.get_enterprise_catalog(catalog_uuid=catalog_uuid)
    except NotConnectedToOpenEdX as exc:
        logger.exception('Unable to update Enterprise Catalog {}'.format(str(catalog_uuid)), exc)
    else:
        if not response:
            # catalog with matching uuid does NOT exist in enterprise-catalog
            # service, so we should create a new catalog
            catalog_client.create_enterprise_catalog(
                str(catalog_uuid),
                str(instance.enterprise_customer.uuid),
                instance.enterprise_customer.name,
                instance.title,
                instance.content_filter,
                instance.enabled_course_modes,
                instance.publish_audit_enrollment_urls,
            )
        else:
            # catalog with matching uuid does exist in enterprise-catalog
            # service, so we should update the existing catalog
            update_fields = {
                'enterprise_customer': str(instance.enterprise_customer.uuid),
                'enterprise_customer_name': instance.enterprise_customer.name,
                'title': instance.title,
                'content_filter': instance.content_filter,
                'enabled_course_modes': instance.enabled_course_modes,
                'publish_audit_enrollment_urls': instance.publish_audit_enrollment_urls,
            }
            catalog_client.update_enterprise_catalog(catalog_uuid, **update_fields)


@receiver(post_delete, sender=EnterpriseCustomerCatalog)
def delete_enterprise_catalog_data(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Send deletions of Enterprise Catalogs to the Enterprise Catalog Service.
    """
    catalog_uuid = instance.uuid
    try:
        catalog_client = EnterpriseCatalogApiClient()
        catalog_client.delete_enterprise_catalog(catalog_uuid)
    except NotConnectedToOpenEdX as exc:
        logger.exception('Unable to delete Enterprise Catalog {}'.format(str(catalog_uuid)), exc)


def create_enterprise_enrollment_receiver(sender, instance, **kwargs):     # pylint: disable=unused-argument
    """
    Watches for post_save signal for creates on the CourseEnrollment table.

    Spin off an async task to generate an EnterpriseCourseEnrollment if appropriate.
    """
    if kwargs.get('created') and instance.user:
        user_id = instance.user.id
        try:
            ecu = EnterpriseCustomerUser.objects.get(user_id=user_id)
        except ObjectDoesNotExist:
            return
        logger.info((
            "User %s is an EnterpriseCustomerUser. "
            "Spinning off task to check if course is within User's "
            "Enterprise's EnterpriseCustomerCatalog."
        ), user_id)

        create_enterprise_enrollment.delay(
            str(instance.course_id),
            ecu.id,
        )


# Don't connect this receiver if we dont have access to CourseEnrollment model
if CourseEnrollment is not None:
    post_save.connect(create_enterprise_enrollment_receiver, sender=CourseEnrollment)



# Generated by Django 2.2.16 on 2020-09-30 17:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('blackboard', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='blackboardenterprisecustomerconfiguration',
            name='client_id',
            field=models.CharField(help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls on behalf of the customer. Called Application Key in Blackboard', max_length=255, null=True, verbose_name='API Client ID or Blackboard Application Key'),
        ),
        migrations.AlterField(
            model_name='blackboardenterprisecustomerconfiguration',
            name='client_secret',
            field=models.CharField(help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls on behalf of the customer. Called Application Secret in Blackboard', max_length=255, null=True, verbose_name='API Client Secret or Application Secret'),
        ),
        migrations.AlterField(
            model_name='historicalblackboardenterprisecustomerconfiguration',
            name='client_id',
            field=models.CharField(help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls on behalf of the customer. Called Application Key in Blackboard', max_length=255, null=True, verbose_name='API Client ID or Blackboard Application Key'),
        ),
        migrations.AlterField(
            model_name='historicalblackboardenterprisecustomerconfiguration',
            name='client_secret',
            field=models.CharField(help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls on behalf of the customer. Called Application Secret in Blackboard', max_length=255, null=True, verbose_name='API Client Secret or Application Secret'),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Learner data exporter for Enterprise Integrated Channel Blackboard.
"""

from logging import getLogger

from django.apps import apps

from enterprise.api_client.discovery import get_course_catalog_api_service_client
from integrated_channels.integrated_channel.exporters.learner_data import LearnerExporter
from integrated_channels.utils import parse_datetime_to_epoch_millis

LOGGER = getLogger(__name__)


class BlackboardLearnerExporter(LearnerExporter):
    """
    Class to provide a Blackboard learner data transmission audit prepared for serialization.
    """

    def get_learner_data_records(
            self,
            enterprise_enrollment,
            completed_date=None,
            is_passing=False,
            **kwargs
    ):  # pylint: disable=arguments-differ,unused-argument
        """
        Return a BlackboardLearnerDataTransmissionAudit with the given enrollment and course completion data.
        If completed_date is None, then course completion has not been met.
        If no remote ID can be found, return None.
        """
        enterprise_customer = enterprise_enrollment.enterprise_customer_user
        completed_timestamp = None
        if completed_date is not None:
            completed_timestamp = parse_datetime_to_epoch_millis(completed_date)

        if enterprise_customer.user_email is None:
            LOGGER.debug(
                'No learner data was sent for user [%s] because a Blackboard user ID could not be found.',
                enterprise_customer.username
            )
            return None

        BlackboardLearnerDataTransmissionAudit = apps.get_model(  # pylint: disable=invalid-name
            'blackboard',
            'BlackboardLearnerDataTransmissionAudit'
        )

        course_catalog_client = get_course_catalog_api_service_client(
            site=enterprise_customer.enterprise_customer.site
        )

        percent_grade = kwargs.get('grade_percent', None)

        return [
            BlackboardLearnerDataTransmissionAudit(
                enterprise_course_enrollment_id=enterprise_enrollment.id,
                blackboard_user_email=enterprise_customer.user_email,
                course_id=course_catalog_client.get_course_id(enterprise_enrollment.course_id),
                course_completed=completed_date is not None and is_passing,
                grade=percent_grade,
                completed_timestamp=completed_timestamp,
            ),
            BlackboardLearnerDataTransmissionAudit(
                enterprise_course_enrollment_id=enterprise_enrollment.id,
                blackboard_user_email=enterprise_customer.user_email,
                course_id=enterprise_enrollment.course_id,
                course_completed=completed_date is not None and is_passing,
                grade=percent_grade,
                completed_timestamp=completed_timestamp,
            )
        ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-07-02 13:09


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0017_sapsuccessfactorsglobalconfiguration_search_student_api_path'),
    ]

    operations = [
        migrations.AddField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='show_course_price',
            field=models.BooleanField(default=False),
        ),
    ]



# -*- coding: utf-8 -*-
"""
Class for transmitting content metadata to Moodle.
"""
import logging
from itertools import islice
from time import sleep

from django.conf import settings

from integrated_channels.exceptions import ClientError
from integrated_channels.integrated_channel.transmitters.content_metadata import ContentMetadataTransmitter
from integrated_channels.moodle.client import MoodleAPIClient
from integrated_channels.utils import chunks

# Based on averages and the default nginx url length limit,
# we might as well go to 1 if we hit a failure.
LOW_CHUNK_SIZE = 1

LOGGER = logging.getLogger(__name__)


class MoodleContentMetadataTransmitter(ContentMetadataTransmitter):
    """
    This transmitter transmits exported content metadata to Moodle.
    """

    def __init__(self, enterprise_configuration, client=MoodleAPIClient):
        """
        Use the ``MoodleAPIClient`` for content metadata transmission to Moodle.
        """
        super(MoodleContentMetadataTransmitter, self).__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )

    def _transmit_create(self, channel_metadata_item_map, chunk_size=None, count=0):  # pylint: disable=arguments-differ
        """
        Transmit content metadata creation to integrated channel.
        """
        chunk_size = chunk_size if chunk_size else self.enterprise_configuration.transmission_chunk_size
        chunk_items = chunks(channel_metadata_item_map, chunk_size)
        transmission_limit = settings.INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT.get(
            self.enterprise_configuration.channel_code()
        )
        for chunk in islice(chunk_items, transmission_limit):
            serialized_chunk = self._serialize_items(list(chunk.values()))
            try:
                self.client.create_content_metadata(serialized_chunk)
            except ClientError as exc:
                if exc.status_code == 414 and count <= 1:
                    # On initial 414, we should retry after slight delay using our lowest chunk size.
                    # Since our lowest is currently 1, if we fail again,
                    # we just need to record the exception and give up.
                    sleep(5)
                    count += 1
                    self._transmit_create(channel_metadata_item_map, chunk_size=LOW_CHUNK_SIZE, count=count)
                else:
                    LOGGER.error(
                        'Failed to create [%s] content metadata items for integrated channel [%s] [%s]. '
                        'Task failed with message [%s] and status code [%s]',
                        len(chunk),
                        self.enterprise_configuration.enterprise_customer.name,
                        self.enterprise_configuration.channel_code(),
                        exc.message,
                        exc.status_code
                    )
                    LOGGER.exception(exc)
            else:
                self._create_transmissions(chunk)

    def _transmit_update(self, channel_metadata_item_map, transmission_map, chunk_size=None, count=0):  # pylint: disable=arguments-differ
        """
        Transmit content metadata update to integrated channel.
        """
        chunk_size = chunk_size if chunk_size else self.enterprise_configuration.transmission_chunk_size
        chunk_items = chunks(channel_metadata_item_map, chunk_size)
        transmission_limit = settings.INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT.get(
            self.enterprise_configuration.channel_code()
        )
        for chunk in islice(chunk_items, transmission_limit):
            serialized_chunk = self._serialize_items(list(chunk.values()))
            try:
                self.client.update_content_metadata(serialized_chunk)
            except ClientError as exc:
                if exc.status_code == 414 and count <= 1:
                    count += 1
                    sleep(5)
                    self._transmit_update(
                        channel_metadata_item_map,
                        transmission_map,
                        chunk_size=LOW_CHUNK_SIZE,
                        count=count
                    )
                else:
                    LOGGER.error(
                        'Failed to update [%s] content metadata items for integrated channel [%s] [%s]. '
                        'Task failed with message [%s] and status code [%s]',
                        len(chunk),
                        self.enterprise_configuration.enterprise_customer.name,
                        self.enterprise_configuration.channel_code(),
                        exc.message,
                        exc.status_code
                    )
                    LOGGER.exception(exc)
            else:
                self._update_transmissions(chunk, transmission_map)

    def _transmit_delete(self, channel_metadata_item_map, chunk_size=None, count=0):  # pylint: disable=arguments-differ
        """
        Transmit content metadata deletion to integrated channel.
        """
        chunk_size = chunk_size if chunk_size else self.enterprise_configuration.transmission_chunk_size
        chunk_items = chunks(channel_metadata_item_map, chunk_size)
        transmission_limit = settings.INTEGRATED_CHANNELS_API_CHUNK_TRANSMISSION_LIMIT.get(
            self.enterprise_configuration.channel_code()
        )
        for chunk in islice(chunk_items, transmission_limit):
            serialized_chunk = self._serialize_items(list(chunk.values()))
            try:
                self.client.delete_content_metadata(serialized_chunk)
            except ClientError as exc:
                if exc.status_code == 414 and count <= 1:
                    count += 1
                    sleep(5)
                    self._transmit_delete(channel_metadata_item_map, chunk_size=LOW_CHUNK_SIZE, count=count)
                else:
                    LOGGER.error(
                        'Failed to delete [%s] content metadata items for integrated channel [%s] [%s]. '
                        'Task failed with message [%s] and status code [%s]',
                        len(chunk),
                        self.enterprise_configuration.enterprise_customer.name,
                        self.enterprise_configuration.channel_code(),
                        exc.message,
                        exc.status_code
                    )
                    LOGGER.exception(exc)
            else:
                self._delete_transmissions(chunk.keys())

    def _prepare_items_for_transmission(self, channel_metadata_items):
        """
        Takes items from the exporter and formats the keys in the way required for Moodle.
        """
        items = {}
        for index, item in enumerate(channel_metadata_items):
            for key in item:
                new_key = 'courses[{0}][{1}]'.format(index, key)
                items[new_key] = item[key]
        return items

    def _serialize_items(self, channel_metadata_items):
        """
        Overrides the base class _serialize_items method such that we return an object
        instead of a binary string.
        """
        return self._prepare_items_for_transmission(channel_metadata_items)



# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0014_drop_historical_table'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalsapsuccessfactorsenterprisecustomerconfiguration',
            name='enterprise_customer',
        ),
        migrations.RemoveField(
            model_name='historicalsapsuccessfactorsenterprisecustomerconfiguration',
            name='history_user',
        ),
        migrations.DeleteModel(
            name='HistoricalSAPSuccessFactorsEnterpriseCustomerConfiguration',
        ),
    ]



"""
Content metadata exporter for Canvas
"""

from logging import getLogger

from integrated_channels.integrated_channel.exporters.content_metadata import ContentMetadataExporter

LOGGER = getLogger(__name__)
BLACKBOARD_COURSE_CONTENT_NAME = 'edX Integration'


class BlackboardContentMetadataExporter(ContentMetadataExporter):
    """
        Blackboard implementation of ContentMetadataExporter.
        Note: courseId is not being exported here (instead done in client during content send)
    """
    DATA_TRANSFORM_MAPPING = {
        'externalId': 'key',
        'course_metadata': 'course_metadata',
        'course_content_metadata': 'course_content_metadata',
        'course_child_content_metadata': 'course_child_content_metadata',
    }

    DESCRIPTION_TEXT_TEMPLATE = "<a href={enrollment_url}>Go to edX course page</a><br />"

    def transform_course_metadata(self, content_metadata_item):
        """
        Formats the metadata necessary to create a base course object in Blackboard
        """
        return {
            'name': content_metadata_item.get('title', None),
            'externalId': content_metadata_item.get('key', None),
            'description': self.DESCRIPTION_TEXT_TEMPLATE.format(
                enrollment_url=content_metadata_item.get('enrollment_url', None)
            )
        }

    def transform_course_content_metadata(self, content_metadata_item):  # pylint: disable=unused-argument
        """
        Formats the metadata necessary to create a course content object in Blackboard
        """
        return {
            'title': BLACKBOARD_COURSE_CONTENT_NAME,
            'position': 0,
            "contentHandler": {"id": "resource/x-bb-folder"}
        }

    def transform_course_child_content_metadata(self, content_metadata_item):
        """
        Formats the metadata necessary to create a course content object in Blackboard
        """
        title = content_metadata_item.get('title', None)
        return {
            'title': title,
            'availability': 'Yes',
            'contentHandler': {
                'id': 'resource/x-bb-externallink',
                'url': content_metadata_item.get('enrollment_url', None),
            },
            'body': '<div>{title}</div><div>{description}</div><img src={image_url} />'.format(
                title=title,
                description=content_metadata_item.get('full_description', None),
                image_url=content_metadata_item.get('image_url', None),
            )
        }



# -*- coding: utf-8 -*-
"""
Class for transmitting learner data to Blackboard.
"""

from integrated_channels.blackboard.client import BlackboardAPIClient
from integrated_channels.integrated_channel.transmitters.learner_data import LearnerTransmitter


class BlackboardLearnerTransmitter(LearnerTransmitter):
    """
    This endpoint is intended to receive learner data routed from the integrated_channel app that is ready to be
    sent to Blackboard.
    """

    def __init__(self, enterprise_configuration, client=BlackboardAPIClient):
        """
        By default, use the ``BlackboardAPIClient`` for learner data transmission to Blackboard.
        """
        super(BlackboardLearnerTransmitter, self).__init__(
            enterprise_configuration=enterprise_configuration,
            client=client
        )

    def transmit(self, payload, **kwargs):
        """
        Send a completion status call to Blackboard using the client.

        Args:
            payload: The learner completion data payload to send to Blackboard
        """
        kwargs['app_label'] = 'blackboard'
        kwargs['model_name'] = 'BlackboardLearnerDataTransmissionAudit'
        kwargs['remote_user_id'] = 'blackboard_user_email'
        super(BlackboardLearnerTransmitter, self).transmit(payload, **kwargs)
